/*
 * titraxHist.c
 *
 * Titrax log-database subroutines
 *
 * $Id: titraxHist.c,v 1.1 1999/02/21 13:41:57 ga58 Exp $
 *
 * $Log: titraxHist.c,v $
 * Revision 1.1  1999/02/21 13:41:57  ga58
 *  - Version 3.29
 *
 * Revision 1.2  1999/01/24 20:23:11  ga58
 *  - Beta version 3.23
 *
 * Revision 1.1  1999/01/23 16:02:12  ga58
 *  - checkin before removing all row-constructs
 *
 */

//#pragma pack(2)

#include "titraxAll.h"
#include "titraxConst.h"
#include "slotName.h"
#include "titraxHist.h"
#include <stddef.h>
//#include "titraxDB.h"
//#include "titraxNote.h"

static DmOpenRef TitraxDB;

#define titraxDBName "TitraxHistDB"
#define titraxDBType 'Hist'


/*
 * Database operations
 * 
 * These routines do NOT perform any error-checking
 */
void
CreateHistDB(void)
{
  Err err;
  LocalID dbID;
  UInt16 attr;

  /* check for a existing database */
  dbID = DmFindDatabase(0, titraxDBName);
  if (dbID != 0)
	 DmDeleteDatabase(0, dbID);

  err = DmCreateDatabase(0, titraxDBName, titraxAppType, titraxDBType, false);
  ErrFatalDisplayIf(err, "Could not create database.");

  dbID = DmFindDatabase(0, titraxDBName);

  // this makes it so that the database is backed up to the desktop
  // if no conduits have registered for the responsibility
    err = DmDatabaseInfo (    0,//UInt16 cardNo,
                            dbID,//LocalID dbID,
                            NULL,//Char *nameP,
                            &attr,//UInt16 *attributesP,
                            NULL, //UInt16 *versionP,
                            NULL, //UInt32 *crDateP,
                            NULL, //UInt32 *modDateP,
                            NULL, //UInt32 *bckUpDateP,
                            NULL, //UInt32 *modNumP,
                            NULL, //LocalID *appInfoIDP,
                            NULL, //LocalID *sortInfoIDP,
                            NULL, //UInt32 *typeP,
                            NULL //UInt32 *creatorP    
                            );
    ErrFatalDisplayIf(err, "Could not retrieve DB info.");

    attr |= dmHdrAttrBackup;

    err = DmSetDatabaseInfo ( 0 ,      //UInt16 cardNo,
                            dbID,    //LocalID dbID,
                            NULL, //const Char *nameP,
                            &attr, //UInt16 *attributesP,
                            NULL, //UInt16 *versionP,
                            NULL, //UInt32 *crDateP,
                            NULL, //UInt32 *modDateP,
                            NULL, //UInt32 *bckUpDateP,
                            NULL, //UInt32 *modNumP,
                            NULL, //LocalID *appInfoIDP,
                            NULL, //LocalID *sortInfoIDP,
                            NULL, //UInt32 *typeP,
                            NULL //UInt32 *creatorP
                            );
    ErrFatalDisplayIf(err, "Could not set DB info.");

}

void
OpenHistDB(void)
{
  TitraxDB = DmOpenDatabaseByTypeCreator(titraxDBType, titraxAppType, dmModeReadWrite);
  if (TitraxDB == 0){
	 CreateHistDB();
     TitraxDB = DmOpenDatabaseByTypeCreator(titraxDBType, titraxAppType, dmModeReadWrite);
  }

  if (TitraxDB == 0){
      ErrFatalDisplayIf(0, "Couldn't open DB even after creation.");
  }

}

void
CloseHistDB(void)
{
  DmCloseDatabase(TitraxDB);
}

Int16
HistCompareRecords(histTypePtr r1, histTypePtr r2, Int16 extra,
						 SortRecordInfoPtr info1, SortRecordInfoPtr info2,
						 MemHandle appInfoH)
{
  Int16 result;

  if (r1->start < r2->start)
	 result = -1;
  else if (r1->start > r2->start)
	 result = 1;
  else {
	 if (r1->stop < r2->stop)
		result = -1;
	 else if (r1->stop > r2->stop)
		result = 1;
	 else
		result = 0;
  }

  return result;
}

void
AddLogEntry(Int16 slot, UInt32 start, UInt32 stop)
{
  UInt16 idx;
  UInt32 size;
  char *name;
  MemHandle recH;
  histTypePtr recP;


  //HostTraceOutputTL(errNone,"Entering AddLogEntry(%u,%lu,%lu)",slot,start,stop);



  OpenHistDB();

  size = sizeof(histType);

  recH = DmNewHandle(TitraxDB, size);
  if (recH == NULL)
	 ErrDisplay("HistCreateRecord: could not create handle.");

  recP = (histTypePtr)MemHandleLock(recH);
  DmSet(recP, 0, size, 0);

  name = SlotName(slot);

  DmStrCopy(recP, 0, name);
  //HostTraceOutputTL(errNone,"recP->name %s",recP->name);
  //HostTraceOutputTL(errNone,"sizeof(histType) %lu",sizeof(histType));


  //HostTraceOutputTL(errNone,"right before DmWrite %lu %lu",start,stop);

  // FUCKING BYTE ALIGNMENT. JESUS! That's 2 days lost.
  //HostTraceOutputTL(errNone,"recP %lu, &(recP->start) %lu,&(recP->stop) %lu",recP, &(recP->start),&(recP->stop));

  //HostTraceOutputTL(errNone," (void*)&(recP->start) - (void*)recP %lu",
  //                            (void*)&(recP->start) - (void*)recP);
  DmWrite((void*)recP, (void*)&(recP->start) - (void*)recP, (void*)&start, sizeof(start));
  DmWrite((void*)recP, (void*)&(recP->stop) - (void*)recP, (void*)&stop, sizeof(stop));

  //HostTraceOutputTL(errNone,"offsets %lu %lu",OffsetOf(histType,start),OffsetOf(histType,stop));
  //HostTraceOutputTL(errNone,"start/stop after DmWrite %lu %lu",start,stop);
  //HostTraceOutputTL(errNone,"recP after DmWrite %lu %lu",(recP->start),(recP->stop));

  idx = DmFindSortPosition(TitraxDB, recP, NULL, (DmComparF*)HistCompareRecords, 0);

  MemPtrUnlock(recP);


  if (DmAttachRecord(TitraxDB, &idx, (MemHandle)recH, 0)) {
	 MemHandleFree(recH);
	 ErrDisplay("AddLogEntry: could not attach record.");
  }

  CloseHistDB();
}

